<?php
class Api {

	private $url;
    private $token;

	public function __construct($url, $token) {
        if (!$url) { throw new Exception('Api URL is required'); }
        if (!$token) { throw new Exception('Api token is required'); }
        $this->url = rtrim($url, '/');
        $this->token = $token;
	}
	
	// API methods
	public function get($endpoint, $filters = array()) {
		return $this->_request('GET', $endpoint, $filters);
	}
	public function post($endpoint, $data) {
        if (!is_array($data) || count($data) == 0) { throw new Exception('Data input variable is required'); }
		return $this->_request('POST', $endpoint, "", $data);
	}
	public function put($endpoint, $data, $filters = array()) {
        if (!is_array($data) || count($data) == 0) { throw new Exception('Data input variable is required'); }
		return $this->_request('PUT', $endpoint, $filters, $data);
	}
	public function delete($endpoint, $filters = array()) {
		return $this->_request('DELETE', $endpoint, $filters);
	}
	// ---
	
	// Execute the API request
	private function _request($method, $endpoint, $filters, $data = "") {
		$dataString = json_encode($data);
		$url = $this->url . '/' . rtrim($endpoint, '/?&') . "/?token=" . $this->token . $this->getFilterEncodedUrl($filters);
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		if ($method != "GET") {
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
		}
		if ($method == "POST" || $method == "PUT") {
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $dataString);
			curl_setopt($ch, CURLOPT_HTTPHEADER, array(                                                                          
				'Content-Type: application/json',                                                        
				'Content-Length: ' . strlen($dataString)                                                                       
			));    
		}
		curl_setopt($ch, CURLOPT_HEADER, true);

		// Execute the request
		$result = curl_exec($ch);

		// Get result data
		$headerSize = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
		$output['http_code'] = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		$output['header'] = $this->getHeaders(substr($result, 0, $headerSize));
		$output['body'] = substr($result, $headerSize);		
	
		// Close curl resource to free up system resources
		curl_close($ch);
		
		// Return data
		return $output;
	}

	// Translating headers to an array
	private function getHeaders($headers) {

		$data = explode("\n", $headers);
		$return = array();
		
		// Split http_status
		$return['status'] = $data[0];
		array_shift($data);
		
		foreach($data as $header) {
			$name = substr($header, 0, strpos($header, ":"));
			$value = substr($header, strpos($header, ":") + strlen(":"));
			$return[$name] = trim($value);
		}
		
		return $return;
	}
	
	// Generate filter syntax
	private function getFilterEncodedUrl($filters) {
		$return = "";
		if (is_array($filters)) {
			if (count($filters) > 0) {
				foreach($filters as $filter) {
					if ($filter) { $return .= "&filter=" . ltrim(urlencode($filter), '&'); }
				}
			}
		} else {
			if ($filters) { $return .= "&filter=" . ltrim(urlencode($filters), '&'); }
		}
		return $return;
	}

}
?>